@@ -48,6 +48,8 @@
 /* for free() */
 #include <stdlib.h>
 
+#include "assert.h"
+
 extern void *xmalloc (size_t size);
 extern void *xzmalloc (size_t size);
 extern void *xcalloc (size_t nmemb, size_t size);
@@ -58,14 +60,14 @@ extern char *xstrdupsafe (const char *str);
 extern char *xstrndup (const char *str, size_t len);
 extern void *xmemdup (const void *ptr, size_t size);
 
-// this can be used on lvalues (e.g. function returns)
-#define xfree(ptr) do { \
-  void *x = ptr; if (x) { free(x); } \
-} while (0)
-
-// NOTE - can not be used on things that can't be lvalues :)
-#define xzfree(ptr) do { \
-  if (ptr) { free(ptr); ptr = NULL; } \
+/* free a pointer and also zero it
+ *
+ * CAUTION: ptr argument is evaluated multiple times, beware side effects!
+ */
+#define xzfree(ptr) do {    \
+    assert((ptr) == (ptr)); \
+    free(ptr);              \
+    (ptr) = NULL;           \
 } while (0)
 
 /* Functions using xmalloc.h must provide a function called fatal() conforming
